#!/usr/bin/env node
const path = require('path')
const fs = require('fs-extra')

const { kebabCase } = require('./src/utils/case')

const rollup = require('./src/rollup')
const md2vue = require('./src/md2vue')
const NuxtDevServer = require('./src/nuxt-server')
const NuxtGenerator = require('./src/nuxt-generate')
const createBoilerplate = require('./src/boilerplate')
const startComponentDev = require('./src/single-comp')

// check config file
const conf = path.join(process.cwd(), 'clair.config.js')
if (!fs.existsSync(conf)) {
  console.error(
    'No such file called `clair.config.js` in' +
    'current working directory'
  )
  process.exit()
}

// try to require config file
let config = null
try {
  config = require(conf)
} catch (e) {
  console.error(e)
  process.exit()
}

const argv = process.argv.slice(2)
switch (argv[0]) {
  case 'start':
    process.env.PORT = 1126
    process.env.NODE_ENV = 'development'
    Promise.resolve()
      .then(() => rollup.watchBuild(config.rollup))
      .then(() => md2vue.watchBuild(config.md2vue))
      .then(() => NuxtDevServer(config.nuxt))
      .catch(e => {
        console.log(e)
        process.exit()
      })
    break

  case 'build':
    process.env.NODE_ENV = 'production'
    Promise.resolve()
      .then(() => rollup.build(config.rollup))
      .then(() => md2vue.build(config.md2vue))
      .then(() => NuxtGenerator(config.nuxt))
      .catch(e => {
        console.log(e)
        process.exit()
      })
    break

  case 'add':
    if (argv[1]) {
      const obj = Object.assign({
        name: argv[1],
        prefix: 'c',
        dir: 'src/components'
      }, config.boilerplate)

      createBoilerplate(obj)
    }
    break

  case 'dev':
    if (argv[1]) {
      const { dir } = config.boilerplate
      const component = kebabCase(argv[1])
      const path = `${dir}/${component}/`
      let promise = Promise.resolve()

      if (fs.pathExistsSync(path) === false) {
        const obj = Object.assign({
          name: argv[1],
          prefix: 'c',
          dir: 'src/components'
        }, config.boilerplate)

        promise = promise.then(() => createBoilerplate(obj))
      }

      promise.then(() =>
        startComponentDev(config, `${path}/index.md`)
      )
    }
    break
  default:
}
