/* eslint-disable no-new-func */
/**
 * markdown document to nuxt.js vue page
 */
const path = require('path')
const glob = require('glob')
const fs = require('fs-extra')
const chokidar = require('chokidar')

const compileFromMd = require('./utils/compile')

/**
 * build once
 */
exports.build = function ({ globs, output, vueTools, highlight }) {
  console.log(`[md2vue] release-build starts.`)

  const ret = globs
    .reduce((acc, pattern) => acc.concat(glob.sync(pattern)), [])
    .forEach(file => compile2vue({ file, output, vueTools, highlight }))

  console.log(`[md2vue] release-build done.`)

  return ret
}

/**
 * watch mode
 */
exports.watchBuild = function ({ globs, output, vueTools, highlight }) {
  console.log(`[md2vue] watch-build starts.`)

  return chokidar.watch(globs, {
    persistent: true
  })
  .on('add', file => {
    console.log(`[md2vue] ${file} added.`)
    compile2vue({ file, output, vueTools, highlight })
  })
  .on('change', file => {
    console.log(`[md2vue] ${file} has been changed.`)
    compile2vue({ file, output, vueTools, highlight })
  })
}

function compile2vue (opt) {
  compileFromMd(opt)
    .then(([code, route]) => {
      const dest = path.join(opt.output, `${route}.vue`)
      return fs.ensureFile(dest)
        .then(() => fs.writeFile(dest, code))
    })
    .catch(e => console.log(e))
}
