const rollup = require('rollup')
const chokidar = require('chokidar')
const fs = require('fs-extra')

exports.build = function ({ name, input, output, plugins }) {
  console.log(`[Rollup] release-build starts.`)

  // returns a Promise
  return rollup.rollup({
    input,
    plugins
  })
  .then(bundle =>
    Promise.all(
      output.map(({ file, format }) => {
        fs.ensureFileSync(file)
        return bundle.write({ name, file, format })
      })
    )
  )
  .then(ret => {
    console.log(`[Rollup] release-build done.`)
    return ret
  })
  .catch(e => {
    console.log(e)
  })
}

exports.watchBuild = function ({ name, input, output, plugins }) {
  console.log(`[Rollup] watch-build starts.`)

  // https://rollupjs.org/#rollup-watch
  const watcher = rollup.watch({
    input,
    plugins,
    output: output.map(o => (o.name = name) && o),
    watch: { chokidar }
  })

  return new Promise((resolve, reject) => {
    watcher.on('event', function listener ({ code, error }) {
      if (error) {
        reject(error)
      }

      if (code === 'END') {
        console.log(`[Rollup]: build done.`)

        watcher.removeListener('event', listener)
        watcher.on('event', ({ code, err }) => {
          if (err) {
            console.log(`[Rollup]: Error ${err}`)
          } else {
            console.log(`[Rollup]: ${code}`)
          }
        })
        resolve()
      }
    })
  })
}
