const path = require('path')
const fs = require('fs-extra')
const browserSync = require('browser-sync')

const compileFromMd = require('./utils/compile')
const rollup = require('./rollup')

let compiledCode = ''
const template = code => `<!DOCTYPE html>
<html>
  <head>
    <meta charset="utf-8">
    <meta name="viewport" content="width=device-width">
    <title>Clair</title>
    <link href="https://lib.baomitu.com/font-awesome/4.7.0/css/font-awesome.min.css" rel="stylesheet">
    <link href="/dist/clair.css" rel="stylesheet">
    <link href="/docs/assets/css/main.css" rel="stylesheet">
    <style>body{max-width:1000px;margin:auto;}</style>
  </head>
  <body>
  <div id="app"></div>
  <script src="http://lib.baomitu.com/vue/2.2.6/vue.js"></script>
  <script src="/dist/clair.js"></script>
  <script>
  process = {};
  process.BROWSER_BUILD = true;
  </script>
  <script>${code}</script>
  <script>
  Vue.use(Clair);
  new Vue({
    el: '#app',
    template: '<devapp />'
  });
  </script>
  </body>
</html>`

/**
 * watch mode
 */
module.exports = function startComponentDev (config, file) {
  // console.log(`[md2vue] watch-build starts.`)
  const compile = () => {
    return compile2vue(file).then(([code]) => {
      compiledCode = code
    })
  }

  compile().then(() => {
    const server = bsServe()

    server.watch(file).on('change', () => {
      compile().then(() => server.reload())
    })
  })

  rollup.watchBuild(config.rollup)
}

function bsServe () {
  const server = browserSync.create()

  server.init({
    server: true,
    files: [
      'dist/*',
      'docs/assets/css'
    ],
    port: 8080,
    middleware: [middleware],
    open: 'external',
    notify: false,
    reloadThrottle: 80,
    ghostMode: false
  })

  return server
}

function middleware (req, res, next) {
  if (req.url !== '/') {
    const file = `./${req.url}`

    if (fs.pathExistsSync(file)) {
      return next()
    }

    const relative = path.relative('docs/assets/css', file)
    const depFile = path.join('node_modules', relative)

    if (fs.pathExistsSync(depFile)) {
      return fs.createReadStream(depFile).pipe(res)
    }

    res.writeHead(404)
    res.end()
  } else {
    res.end(template(compiledCode))
  }
}

function compile2vue (file) {
  return compileFromMd({
    file,
    target: 'js',
    componentName: 'devapp',
    vueTools: defaultVueTools
  })
}

function defaultVueTools (name, uid) {
  return `<input id="${uid}" type="checkbox" tabindex="-1" aria-hidden="true"/>
  <label for="${uid}" aria-hidden="true"></label>`
}
