/* eslint-disable no-new-func */
const path = require('path')

const fs = require('fs-extra')
const md2vue = require('md2vue')
const yamlFront = require('yaml-front-matter')

module.exports = function ({
  file,
  vueTools,
  componentName,
  target = 'vue',
  highlight = 'highlight.js'
}) {
  // src/components/button/index.md => button => button.vue
  const name = path.basename(path.dirname(file))
  const defaults = {
    title: 'Component',
    layout: 'component',
    scrollTop: true,
    route: `component/${name}`
  }

  return fs.readFile(file)
    .then(buf => getFrontMatter(buf.toString()))
    .then(({ source, config }) => {
      const { layout, scrollTop, title, route } = Object.assign(defaults, config)
      const documentInfo = {
        layout: layout,
        scrollTop: scrollTop,
        head: new Function(`return { title: "${title}" }`)
      }

      let id = 0
      const customMarkups = function () {
        const type = typeof vueTools
        const uid = `demo-${route}-${id++}`

        if (type === 'string') {
          return vueTools
        }

        if (type === 'function') {
          return vueTools(name, uid)
        }

        return ''
      }

      return md2vue(source, {
        target,
        customMarkups,
        documentInfo,
        highlight,
        componentName
      }).then(code => {
        return [code, route]
      })
    })
}

/**
 * 获取 markdown 文件的 yaml 信息
 */
function getFrontMatter (source) {
  const result = yamlFront.loadFront(source, '__mdContent')
  const { __mdContent } = result
  delete result.__mdContent

  return {
    source: __mdContent,
    config: result
  }
}
